@@ -5,6 +5,7 @@
 .. autosummary::
    
     ~AD_setup_FrameType
+    ~AD_warmed_up
     ~ApsHDF5Plugin
     ~ApsMachineParametersDevice
     ~ApsPssShutter
@@ -26,6 +27,7 @@
     ~swait_setup_gaussian
     ~swait_setup_lorentzian
     ~swait_setup_incrementer
+    ~TunableEpicsMotor
     ~use_EPICS_scaler_channels
     ~userCalcsDevice
 
@@ -57,6 +59,8 @@ import threading
 import time
 
 from .synApps_ophyd import *
+from . import plans as APS_plans
+
 import ophyd
 from ophyd import Component, Device, DeviceStatus, FormattedComponent
 from ophyd import Signal, EpicsMotor, EpicsSignal, EpicsSignalRO
@@ -128,11 +132,33 @@ class ApsMachineParametersDevice(Device):
         sd = SupplementalData()
         RE.preprocessors.append(sd)
 
+    .. autosummary::
+    
+        ~inUserOperations
+   
+
     """
     current = Component(EpicsSignalRO, "S:SRcurrentAI")
     lifetime = Component(EpicsSignalRO, "S:SRlifeTimeHrsCC")
     machine_status = Component(EpicsSignalRO, "S:DesiredMode", string=True)
+    # In [3]: APS.machine_status.enum_strs
+    # Out[3]: 
+    # ('State Unknown',
+    #  'USER OPERATIONS',
+    #  'Bm Ln Studies',
+    #  'INJ Studies',
+    #  'ASD Studies',
+    #  'NO BEAM',
+    #  'MAINTENANCE')
     operating_mode = Component(EpicsSignalRO, "S:ActualMode", string=True)
+    # In [4]: APS.operating_mode.enum_strs
+    # Out[4]: 
+    # ('State Unknown',
+    # 'NO BEAM',
+    # 'Injecting',
+    # 'Stored Beam',
+    # 'Delivered Beam',
+    # 'MAINTENANCE')
     shutter_permit = Component(EpicsSignalRO, "ACIS:ShutterPermit", string=True)
     fill_number = Component(EpicsSignalRO, "S:FillNumber")
     orbit_correction = Component(EpicsSignalRO, "S:OrbitCorrection:CC")
@@ -140,6 +166,30 @@ class ApsMachineParametersDevice(Device):
     global_feedback_h = Component(EpicsSignalRO, "SRFB:GBL:HLoopStatusBI", string=True)
     global_feedback_v = Component(EpicsSignalRO, "SRFB:GBL:VLoopStatusBI", string=True)
     operator_messages = Component(ApsOperatorMessagesDevice)
+    
+    @property
+    def inUserOperations(self):
+        """
+        determine if APS is in User Operations mode (boolean)
+        
+        Use this property to configure ophyd Devices for direct or simulated hardware.
+        See issue #49 (https://github.com/BCDA-APS/APS_BlueSky_tools/issues/49) for details.
+        
+        EXAMPLE::
+        
+            APS = APS_BlueSky_tools.devices.ApsMachineParametersDevice(name="APS")
+            
+            if APS.inUserOperations:
+                suspend_APS_current = bluesky.suspenders.SuspendFloor(APS.current, 2, resume_thresh=10)
+                RE.install_suspender(suspend_APS_current)
+            else:
+                # use pseudo shutter controls and no current suspenders
+                pass
+
+        """
+        verdict = self.machine_status.value in (1, "USER OPERATIONS")
+        # verdict = verdict and self.operating_mode.value not in (5, "MAINTENANCE")
+        return verdict
 
 
 class ApsPssShutter(Device):
@@ -297,9 +347,11 @@ class ApsPssShutterWithStatus(Device):
         super().__init__(prefix, *args, **kwargs)
 
     def open(self, timeout=10):
+        " "
         ophyd.status.wait(self.set(self.open_str), timeout=timeout)
 
     def close(self, timeout=10):
+        " "
         ophyd.status.wait(self.set(self.close_str), timeout=timeout)
 
     def set(self, value, **kwargs):
@@ -335,10 +387,12 @@ class ApsPssShutterWithStatus(Device):
 
     @property
     def isOpen(self):
+        " "
         return self.pss_state.value == self.open_val
     
     @property
     def isClosed(self):
+        " "
         return self.pss_state.value == self.close_val 
 
 
@@ -439,6 +493,27 @@ class AxisTunerMixin(object):
                 self.post_tune_method()
 
 
+class TunableEpicsMotor(EpicsMotor, AxisTunerMixin):
+    """
+    EPICS motor with signal for tuning
+    
+    USAGE::
+
+        def a2r_pretune_hook():
+            # set the counting time for *this* tune
+            yield from bps.abs_set(scaler.preset_time, 0.2)
+    
+        a2r = TunableEpicsMotor("xxx:m1", name="a2r")
+        a2r.tuner = TuneAxis([scaler], a2r, signal_name=scaler.channels.chan2.name)
+        a2r.tuner.width = 0.02
+        a2r.tuner.num = 21
+        a2r.pre_tune_method = a2r_pretune_hook
+        RE(a2r.tune())
+    
+    """
+    pass
+
+
 class EpicsMotorDialMixin(object):
     """
     add motor record's dial coordinate fields
@@ -589,10 +664,12 @@ class EpicsMotorShutter(Device):
     
     @property
     def isOpen(self):
+        " "
         return abs(self.motor.position - self.open_position) <= self._tolerance
     
     @property
     def isClosed(self):
+        " "
         return abs(self.motor.position - self.closed_position) <= self._tolerance
     
     def open(self):
@@ -857,6 +934,31 @@ def AD_setup_FrameType(prefix, scheme="NeXus"):
         epics.caput(template.format(prefix, "", field), value)
         epics.caput(template.format(prefix, "_RBV", field), value)
 
+         
+def AD_warmed_up(detector):
+    """
+    Has area detector pushed an NDarray to the HDF5 plugin?  True or False
+
+    Works around an observed issue: #598
+    https://github.com/NSLS-II/ophyd/issues/598#issuecomment-414311372
+
+    If detector IOC has just been started and has not yet taken an image
+    with the HDF5 plugin, then a TimeoutError will occur as the
+    HDF5 plugin "Capture" is set to 1 (Start).  In such case,
+    first acquire at least one image with the HDF5 plugin enabled.
+    """
+    old_capture = detector.hdf1.capture.value
+    old_file_write_mode = detector.hdf1.file_write_mode.value
+    if old_capture == 1:
+        return True
+    
+    detector.hdf1.file_write_mode.put(1)
+    detector.hdf1.capture.put(1)
+    verdict = detector.hdf1.capture.get() == 1
+    detector.hdf1.capture.put(old_capture)
+    detector.hdf1.file_write_mode.put(old_file_write_mode)
+    return verdict
+
 
 class ApsFileStoreHDF5(FileStorePluginBase):
     """
@@ -881,9 +983,9 @@ class ApsFileStoreHDF5(FileStorePluginBase):
     ====================================  ============================
     custom class                          superclass(es)
     ====================================  ============================
-    ``ApsFileStoreHDF5``                  ``FileStorePluginBase`
-    ``ApsFileStoreHDF5IterativeWrite``    ``ApsFileStoreHDF5`, `FileStoreIterativeWrite`
-    ``ApsHDF5Plugin``                     ``HDF5Plugin`, `ApsFileStoreHDF5IterativeWrite`
+    ``ApsFileStoreHDF5``                  ``FileStorePluginBase``
+    ``ApsFileStoreHDF5IterativeWrite``    ``ApsFileStoreHDF5``, `FileStoreIterativeWrite``
+    ``ApsHDF5Plugin``                     ``HDF5Plugin``, `ApsFileStoreHDF5IterativeWrite``
     ====================================  ============================
     """
 
@@ -914,7 +1016,7 @@ class ApsFileStoreHDF5(FileStorePluginBase):
         return filename, read_path, write_path
 
     def generate_datum(self, key, timestamp, datum_kwargs):
-        "Generate a uid and cache it with its key for later insertion."
+        """Generate a uid and cache it with its key for later insertion."""
         template = self.file_template.get()
         filename, read_path, write_path = self.make_filename()
         file_number = self.file_number.get()
@@ -927,9 +1029,11 @@ class ApsFileStoreHDF5(FileStorePluginBase):
         return super().generate_datum(key, timestamp, datum_kwargs)
 
     def get_frames_per_point(self):
+        """ """
         return self.num_capture.get()
 
     def stage(self):
+        """ """
         # Make a filename.
         filename, read_path, write_path = self.make_filename()
 
@@ -957,8 +1061,8 @@ class ApsFileStoreHDF5(FileStorePluginBase):
         self._fn = template % (read_path, filename, file_number)
         self._fp = read_path
         if not self.file_path_exists.get():
-            raise IOError("Path %s does not exist on IOC."
-                          "" % self.file_path.get())
+            raise IOError("Path {} does not exist on IOC.".format(
+                          self.file_path.get()))
 
         # from FileStoreIterativeWrite.stage()
         self._point_counter = itertools.count()
